/*
 * Z = exlinesum(S,L,x)
 *
 *  accumulates line shapes for chemical exchange spectra
 *  over the x-axis range given in the vector x, using S(k)
 *  and L(k) to compute real(-S./(L-1i*x) for each line.
 *
 *   S: N-element vector or matrix of complex line amplitudes
 *   L: N-element vector or matrix of complex line position and width
 *   x: m-element vector characterizing the field or frequency range, real
 *
 *   Z: m-element vector containing the sum of all N lines
 *
 */

#include <mex.h>

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{

  /* input parameters */
  double *rS, *iS, *rL, *iL, *omega, *Z;
  /* working variables */
  int nLines, nLinesS, nLinesL, nPoints, idx, iLine;
  double x, rSrL, rLrL, iS_, iL_;
  double *zz;

  /* Argument checks. */
  if (nrhs!=3) mexErrMsgTxt("Wrong number of input parameters!");
  if (nlhs!=1) mexErrMsgTxt("Wrong number of output parameters!");

  /* Get vector lengths. */
  nLinesS = (int)mxGetNumberOfElements(prhs[0]);
  nLinesL = (int)mxGetNumberOfElements(prhs[1]);
  if (nLinesS!=nLinesL)
    mexErrMsgTxt("S and L must have the same number of elements!");
  nLines = nLinesS;

  /* allocate results array */
  nPoints = (int)mxGetNumberOfElements(prhs[2]);
  plhs[0] = mxCreateDoubleMatrix(1,nPoints,mxREAL); 
  Z = mxGetPr(plhs[0]);
    
  /* Get all input arguments. */
  rS = mxGetPr(prhs[0]);
  iS = mxGetPi(prhs[0]);
  rL = mxGetPr(prhs[1]);
  iL = mxGetPi(prhs[1]);
  omega = mxGetPr(prhs[2]);

  /* Z = real(-S./(L-1i*x)); */
  if (iS) { /* if S is complex */
    for (iLine=0;iLine<nLines;iLine++){
      if (mxIsInf(rS[iLine])||mxIsNaN(rS[iLine])) continue;
      rLrL = rL[iLine]*rL[iLine];
      rSrL = rS[iLine]*rL[iLine];
      iL_ = iL[iLine];
      iS_ = iS[iLine];
      for (idx=0;idx<nPoints;idx++){
        x = iL_ + omega[idx];
        Z[idx] -= (rSrL+iS_*x)/(rLrL+x*x);
      }
    }
  }
  else { /* In fast exchange regime, S is real */
    for (iLine=0;iLine<nLines;iLine++){
      if (mxIsInf(rS[iLine])||mxIsNaN(rS[iLine])) continue;
      rLrL = rL[iLine]*rL[iLine];
      rSrL = rS[iLine]*rL[iLine];
      iL_ = iL[iLine];
      for (idx=0;idx<nPoints;idx++){
        x = iL_ + omega[idx];
        Z[idx] -= rSrL/(rLrL+x*x);
      }
    }
  }

} /* void mexFunction */
